@@ -1,4 +1,4 @@
-# Copyright 2020 The ML Collections Authors.
+# Copyright 2021 The ML Collections Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,12 +23,14 @@ configurations of experiments and models.
 
 import abc
 import collections
+from collections import abc as collections_abc
 import contextlib
 import difflib
 import functools
 import inspect
 import json
 import operator
+from typing import Any, Mapping, Optional
 
 from absl import logging
 
@@ -62,6 +64,11 @@ class JSONDecodeError(Exception):
 _NoneType = type(None)
 
 
+def _is_callable_type(field_type):
+  """Tries to ensure: `_is_callable_type(type(obj)) == callable(obj)`."""
+  return any('__call__' in c.__dict__ for c in field_type.__mro__)
+
+
 def _is_type_safety_violation(value, field_type):
   """Helper function for type safety exceptions.
 
@@ -78,8 +85,11 @@ def _is_type_safety_violation(value, field_type):
   # Allow None to override and be overridden by any type.
   if value is None or field_type == _NoneType:
     return False
+  elif isinstance(value, field_type):
+    return False
   else:
-    return not isinstance(value, field_type)
+    # A callable can overridde a callable.
+    return not (callable(value) and _is_callable_type(field_type))
 
 
 def _safe_cast(value, field_type, type_safe=False):
@@ -157,30 +167,29 @@ class _Op(collections.namedtuple('_Op', ['fn', 'args'])):
 
 
 @functools.total_ordering
-class FieldReference(object):
+class FieldReference:
   """Reference to a configuration element.
 
-  Typed configuration element that can take a None default value. Example:
+  Typed configuration element that can take a None default value. Example::
 
-      from ml_collections import config_dict
+    from ml_collections import config_dict
 
-      cfg_field = config_dict.FieldReference(0)
-      cfg = config_dict.ConfigDict({
-          'optional': configdict.FieldReference(None, field_type=str)
-          'field': cfg_field,
-          'nested': {'field': cfg_field}
-      })
+    cfg_field = config_dict.FieldReference(0)
+    cfg = config_dict.ConfigDict({
+        'optional': configdict.FieldReference(None, field_type=str)
+        'field': cfg_field,
+        'nested': {'field': cfg_field}
+    })
 
-      with self.assertRaises(TypeError):
-        cfg.optional = 10  # Raises an error because it's defined as an
-                           # intfield.
+    with self.assertRaises(TypeError):
+      cfg.optional = 10  # Raises an error because it's defined as an
+                          # intfield.
 
-      cfg.field = 1  # Changes the value of both cfg.field and cfg.nested.field.
-      print(cfg)
+    cfg.field = 1  # Changes the value of both cfg.field and cfg.nested.field.
+    print(cfg)
 
-  This class also supports lazy computation. Example:
+  This class also supports lazy computation. Example::
 
-    ```python
     ref = config_dict.FieldReference(0)
 
     # Using ref in a standard operation returns another FieldReference. The new
@@ -193,7 +202,6 @@ class FieldReference(object):
 
     ref.set(-2)  # change ref's value again
     print(ref_plus_ten.get())  # Prints 8 because ref's value is -2
-    ```
   """
 
   def __init__(self, default, field_type=None, op=None, required=False):
@@ -387,6 +395,9 @@ class FieldReference(object):
   def identity(self):
     return self._apply_op(lambda x: x)
 
+  def attr(self, attr_name):
+    return self._apply_op(operator.attrgetter(attr_name))
+
   def __add__(self, other):
     return self._apply_op(operator.add, other)
 
@@ -409,10 +420,10 @@ class FieldReference(object):
     return self._apply_op(rmul)
 
   def __div__(self, other):
-    return self._apply_op(operator.div, other)
+    return self._apply_op(operator.truediv, other)
 
   def __rdiv__(self, other):
-    rdiv = functools.partial(operator.div, other)
+    rdiv = functools.partial(operator.truediv, other)
     return self._apply_op(rdiv)
 
   def __truediv__(self, other):
@@ -548,7 +559,7 @@ def _configdict_fill_seed(seed, initial_dictionary, visit_map=None):
     seed.__setattr__(key, value)
 
 
-class ConfigDict(object):
+class ConfigDict:
   # pylint: disable=line-too-long
   """Base class for configuration objects used in DeepMind.
 
@@ -558,7 +569,7 @@ class ConfigDict(object):
   - it has dot-based access as well as dict-style key access,
   - it is type safe (once a value is set one cannot change its type).
 
-  Typical usage eaxmple:
+  Typical usage example::
 
     from ml_collections import config_dict
 
@@ -571,23 +582,23 @@ class ConfigDict(object):
 
     print(cfg)
 
-  Config dictionaries can also be used to pass named arguments to functions:
+  Config dictionaries can also be used to pass named arguments to functions::
 
-      from ml_collections import config_dict
+    from ml_collections import config_dict
 
-      def print_point(x, y):
-        print "({},{})".format(x, y)
+    def print_point(x, y):
+      print "({},{})".format(x, y)
 
-      point = config_dict.ConfigDict()
-      point.x = 1
-      point.y = 2
-      print_point(**point)
+    point = config_dict.ConfigDict()
+    point.x = 1
+    point.y = 2
+    print_point(**point)
 
   Note that, depending on your use case, it may be easier to use the `create`
-  function in this package to construct a `ConfigDict`:
+  function in this package to construct a `ConfigDict`::
 
-     from ml_collections.config_dict import config_dict
-     point = config_dict.create(x=1, y=2)
+    from ml_collections.config_dict import config_dict
+    point = config_dict.create(x=1, y=2)
 
   Differently from standard `dicts`, `ConfigDicts` also have the nice property
   that iterating over them is deterministic, in a fashion similar to
@@ -595,20 +606,28 @@ class ConfigDict(object):
   """
   # pylint: enable=line-too-long
 
+  # Loosen the static type checking requirements.
+  _HAS_DYNAMIC_ATTRIBUTES = True
+
   def __init__(
-      self, initial_dictionary=None, type_safe=True, convert_dict=True):
+      self,
+      initial_dictionary: Optional[Mapping[str, Any]] = None,
+      type_safe: bool = True,
+      convert_dict: bool = True,
+  ):
     """Creates an instance of ConfigDict.
 
     Warning: In most cases, this faithfully reproduces the reference structure
     of initial_dictionary, even if initial_dictionary is self-referencing.
     However, unexpected behavior occurs if self-references are contained within
-    list, tuple, or custom types. For example:
-        d = {}
-        d['a'] = d
-        d['b'] = [d]
-        cd = ConfigDict(d)
-        cd.a    # refers to cd, type ConfigDict. Expected behavior.
-        cd.b    # refers to d, type dict. Unexpected behavior.
+    list, tuple, or custom types. For example::
+
+      d = {}
+      d['a'] = d
+      d['b'] = [d]
+      cd = ConfigDict(d)
+      cd.a    # refers to cd, type ConfigDict. Expected behavior.
+      cd.b    # refers to d, type dict. Unexpected behavior.
 
     Warning: FieldReference values may be changed. If initial_dictionary
     contains a FieldReference with a value of type dict or FrozenConfigDict,
@@ -616,16 +635,19 @@ class ConfigDict(object):
 
     Args:
       initial_dictionary: May be one of the following:
-          1) dict. In this case, all values of initial_dictionary that are
-             dictionaries are also be converted to ConfigDict. However,
-             dictionaries within values of non-dict type are untouched.
-          2) ConfigDict. In this case, all attributes are uncopied, and only the
-             top-level object (self) is re-addressed. This is the same behavior
-             as Python dict, list, and tuple.
-          3) FrozenConfigDict. In this case, initial_dictionary is converted to
-             a ConfigDict version of the initial dictionary for the
-             FrozenConfigDict (reversing any mutability changes FrozenConfigDict
-             made).
+
+        1) dict. In this case, all values of initial_dictionary that are
+        dictionaries are also be converted to ConfigDict. However,
+        dictionaries within values of non-dict type are untouched.
+
+        2) ConfigDict. In this case, all attributes are uncopied, and only the
+        top-level object (self) is re-addressed. This is the same behavior
+        as Python dict, list, and tuple.
+
+        3) FrozenConfigDict. In this case, initial_dictionary is converted to
+        a ConfigDict version of the initial dictionary for the
+        FrozenConfigDict (reversing any mutability changes FrozenConfigDict
+        made).
       type_safe: If set to True, once an attribute value is assigned, its type
           cannot be overridden without .ignore_type() context manager
           (default: True).
@@ -651,7 +673,7 @@ class ConfigDict(object):
                                           initial_dictionary.is_type_safe)
 
   @property
-  def is_type_safe(self):
+  def is_type_safe(self) -> bool:
     """Returns True if config dict is type safe."""
     return self._type_safe
 
@@ -660,7 +682,7 @@ class ConfigDict(object):
     """Returns True if it is converting dicts to ConfigDict automatically."""
     return self._convert_dict
 
-  def lock(self):
+  def lock(self) -> 'ConfigDict':
     """Locks object, preventing user from adding new fields.
 
     Returns:
@@ -680,11 +702,11 @@ class ConfigDict(object):
     return self
 
   @property
-  def is_locked(self):
+  def is_locked(self) -> bool:
     """Returns True if object is locked."""
     return self._locked
 
-  def unlock(self):
+  def unlock(self) -> 'ConfigDict':
     """Grants user the ability to add new fields to ConfigDict.
 
     In most cases, the unlocked() context manager should be preferred to the
@@ -701,7 +723,7 @@ class ConfigDict(object):
         element.unlock()
     return self
 
-  def get(self, key, default=None):
+  def get(self, key: str, default=None):
     """Returns value if key is present, or a user defined value otherwise."""
     try:
       return self[key]
@@ -725,19 +747,17 @@ class ConfigDict(object):
   def get_oneway_ref(self, key):
     """Returns a one-way FieldReference.
 
-    Example:
+    Example::
 
-    ```
-    cfg = collections.ConfigDict(dict(a=1))
-    cfg.b = cfg.get_oneway_ref('a')
+      cfg = ml_collections.ConfigDict(dict(a=1))
+      cfg.b = cfg.get_oneway_ref('a')
 
-    cfg.a = 2
-    print(cfg.b)  # 2
+      cfg.a = 2
+      print(cfg.b)  # 2
 
-    cfg.b = 3
-    print(cfg.a)  # 2 (would have been 3 if using get_ref())
-    print(cfg.b)  # 3
-    ```
+      cfg.b = 3
+      print(cfg.a)  # 2 (would have been 3 if using get_ref())
+      print(cfg.b)  # 3
 
     Args:
       key: Key for field we want to reference.
@@ -855,7 +875,7 @@ class ConfigDict(object):
                                                              request)
     return message
 
-  def __delitem__(self, key):
+  def __delitem__(self, key: str):
     if self.is_locked:
       raise KeyError('This ConfigDict is locked, you have to unlock it before '
                      'trying to delete a field.')
@@ -872,7 +892,7 @@ class ConfigDict(object):
     except KeyError as e:
       raise KeyError(self._generate_did_you_mean_message(key, str(e)))
 
-  def __getitem__(self, key):
+  def __getitem__(self, key: str):
     if '.' in key:
       # As per the check in __setitem__ above, keys cannot contain dots.
       # Hence, we can use dots to do recursive calls.
@@ -888,15 +908,25 @@ class ConfigDict(object):
     except KeyError as e:
       raise KeyError(self._generate_did_you_mean_message(key, str(e)))
 
-  def __contains__(self, key):
+  def __contains__(self, key: str):
     return key in self._fields
 
-  def __repr__(self):
-    return yaml.dump(self.to_dict(preserve_field_references=True),
-                     default_flow_style=False)
+  def __repr__(self) -> str:
+    # We want __repr__ to always run without throwing an exception,
+    # even if the config dict is not YAML serialisable.
+    try:
+      return yaml.dump(self.to_dict(preserve_field_references=True),
+                       default_flow_style=False)
+    except Exception:  # pylint: disable=broad-except
+      return repr(self.to_dict())
 
-  def __str__(self):
-    return yaml.dump(self.to_dict())
+  def __str__(self) -> str:
+    # We want __str__ to always run without throwing an exception,
+    # even if the config dict is not YAML serialisable.
+    try:
+      return yaml.dump(self.to_dict())
+    except Exception:  # pylint: disable=broad-except
+      return str(self.to_dict())
 
   def keys(self):
     """Returns the sorted list of all the keys defined in a config."""
@@ -963,11 +993,11 @@ class ConfigDict(object):
     """Type-invariant equals.
 
     This is like `__eq__`, except it does not distinguish `FrozenConfigDict`
-    from `ConfigDict`. For example:
+    from `ConfigDict`. For example::
 
-        cd = ConfigDict()
-        fcd = FrozenConfigDict()
-        fcd.eq_as_configdict(cd)  # Returns True
+      cd = ConfigDict()
+      fcd = FrozenConfigDict()
+      fcd.eq_as_configdict(cd)  # Returns True
 
     Args:
       other: Object to compare self to.
@@ -986,6 +1016,23 @@ class ConfigDict(object):
   def to_yaml(self, **kwargs):
     """Returns a YAML representation of the object.
 
+    ConfigDict serializes types of fields as well as the values of fields
+    themselves. Deserializing the YAML representation hence requires using
+    YAML's UnsafeLoader:
+
+    ```
+    yaml.load(cfg.to_yaml(), Loader=yaml.UnsafeLoader)
+    ```
+
+    or equivalently:
+
+    ```
+    yaml.unsafe_load(cfg.to_yaml())
+    ```
+
+    Please see the PyYAML documentation and https://msg.pyyaml.org/load for more
+    details on the consequences of this.
+
     Args:
       **kwargs: Keyword arguments for yaml.dump.
 
@@ -1143,6 +1190,8 @@ class ConfigDict(object):
     """
     visit_map = visit_map or {}
     config_dict_copy = self.__class__()
+    super(ConfigDict, config_dict_copy).__setattr__('_convert_dict',
+                                                    self.convert_dict)
     visit_map[id(self)] = config_dict_copy
 
     for key, value in six.iteritems(self._fields):
@@ -1204,9 +1253,9 @@ class ConfigDict(object):
       if isinstance(field, ConfigDict):
         managers.append(field.ignore_type)
       # Recursively add elements in nested collections.
-      elif isinstance(field, collections.Mapping):
+      elif isinstance(field, collections_abc.Mapping):
         fields.extend(six.iteritems(field))
-      elif isinstance(field, (collections.Sequence, collections.Set)):
+      elif isinstance(field, (collections_abc.Sequence, collections_abc.Set)):
         fields.extend(field)
 
     super(ConfigDict, self).__setattr__('_type_safe', False)
@@ -1260,7 +1309,7 @@ class ConfigDict(object):
       if isinstance(other, ConfigDict):
         iteritems_kwargs['preserve_field_references'] = True
 
-      for key, value in six.iteritems(other, **iteritems_kwargs):
+      for key, value in six.iteritems(other, **iteritems_kwargs):  # pytype: disable=wrong-keyword-args
         if key not in self:
           self[key] = value
         elif isinstance(self._fields[key], ConfigDict):
@@ -1294,55 +1343,55 @@ class ConfigDict(object):
   def update_from_flattened_dict(self, flattened_dict, strip_prefix=''):
     """In-place updates values taken from a flattened dict.
 
-    This allows a potentially nested source `ConfigDict` of the following form:
+    This allows a potentially nested source `ConfigDict` of the following form::
 
-    cfg = ConfigDict({
-        'a': 1,
-        'b': {
-            'c': {
-                'd': 2
-            }
-        }
-    })
+      cfg = ConfigDict({
+          'a': 1,
+          'b': {
+              'c': {
+                  'd': 2
+              }
+          }
+      })
 
     to be updated from a dict containing paths navigating to child items, of the
-    following form:
+    following form::
 
-    updates = {
-        'a': 2,
-        'b.c.d': 3
-    }
+      updates = {
+          'a': 2,
+          'b.c.d': 3
+      }
 
     This filters `paths_dict` to only contain paths starting with
     `strip_prefix` and strips the prefix when applying the update.
 
-    For example, consider we have the following values returned as flags:
+    For example, consider we have the following values returned as flags::
 
-    flags = {
-        'flag1': x,
-        'flag2': y,
-        'config': 'some_file.py',
-        'config.a.b': 1,
-        'config.a.c': 2
-    }
+      flags = {
+          'flag1': x,
+          'flag2': y,
+          'config': 'some_file.py',
+          'config.a.b': 1,
+          'config.a.c': 2
+      }
 
-    config = ConfigDict({
-        'a': {
-            'b': 0,
-            'c': 0
-        }
-    })
+      config = ConfigDict({
+          'a': {
+              'b': 0,
+              'c': 0
+          }
+      })
 
-    config.update_from_flattened_dict(flags, 'config.')
+      config.update_from_flattened_dict(flags, 'config.')
 
-    Then we will now have:
+    Then we will now have::
 
-    config = ConfigDict({
-        'a': {
-            'b': 1,
-            'c': 2
-        }
-    })
+      config = ConfigDict({
+          'a': {
+              'b': 1,
+              'c': 2
+          }
+      })
 
     Args:
       flattened_dict: A mapping (key path) -> value.
@@ -1618,13 +1667,17 @@ class FrozenConfigDict(ConfigDict):
 
     Args:
       initial_dictionary: May be one of the following:
-          1) dict. In this case all values of initial_dictionary that are
-             dictionaries are also converted to FrozenConfigDict. If there are
-             dictionaries contained in lists or tuples, an error is raised.
-          2) ConfigDict. In this case all ConfigDict attributes are also
-             converted to FrozenConfigDict.
-          3) FrozenConfigDict. In this case all attributes are uncopied, and
-             only the top-level object (self) is re-addressed.
+
+        1) dict. In this case all values of initial_dictionary that are
+        dictionaries are also converted to FrozenConfigDict. If there are
+        dictionaries contained in lists or tuples, an error is raised.
+
+        2) ConfigDict. In this case all ConfigDict attributes are also
+        converted to FrozenConfigDict.
+
+        3) FrozenConfigDict. In this case all attributes are uncopied, and
+        only the top-level object (self) is re-addressed.
+
       type_safe: See ConfigDict documentation. Note that this only matters
           if the FrozenConfigDict is converted to ConfigDict at some point.
     """
@@ -1836,23 +1889,23 @@ def create(**kwargs):
   """Creates a `ConfigDict` with the given named arguments as key-value pairs.
 
   This allows for simple dictionaries whose elements can be accessed directly
-  using field access:
+  using field access::
 
-      from ml_collections.config_dict import config_dict
-      point = config_dict.create(x=1, y=2)
-      print(point.x, point.y)
+    from ml_collections.config_dict import config_dict
+    point = config_dict.create(x=1, y=2)
+    print(point.x, point.y)
 
-  This is particularly useful for compactly writing nested configurations:
+  This is particularly useful for compactly writing nested configurations::
 
-      config = config_dict.create(
-          data=config_dict.create(
-              game='freeway',
-              frame_size=100),
-          model=config_dict.create(num_hidden=1000))
+    config = config_dict.create(
+      data=config_dict.create(
+        game='freeway',
+        frame_size=100),
+      model=config_dict.create(num_hidden=1000))
 
   The reason for the existence of this function is that it simplifies the
   code required for the majority of the use cases of `ConfigDict`, compared
-  to using either `ConfigDict` or `namedtuple`s. Examples of such use cases
+  to using either `ConfigDict` or `namedtuple's`. Examples of such use cases
   include training script configuration, and returning multiple named values.
 
   Args:
@@ -1868,11 +1921,11 @@ def create(**kwargs):
 def placeholder(field_type, required=False):
   """Defines an entry in a ConfigDict that has no value yet.
 
-  Example:
+  Example::
 
-      config = configdict.create(
-          batch_size = configdict.placeholder(int),
-          frame_shape = configdict.placeholder(tf.TensorShape))
+    config = configdict.create(
+        batch_size = configdict.placeholder(int),
+        frame_shape = configdict.placeholder(tf.TensorShape))
 
   Args:
     field_type: type of value.
@@ -1888,18 +1941,18 @@ def placeholder(field_type, required=False):
 def required_placeholder(field_type):
   """Defines an entry in a ConfigDict with unknown but required value.
 
-  Example:
+  Example::
 
-      config = configdict.create(
-          batch_size = configdict.required_placeholder(int))
+    config = configdict.create(
+        batch_size = configdict.required_placeholder(int))
 
-      try:
-        print(config.batch_size)
-      except RequiredValueError:
-        pass
+    try:
+      print(config.batch_size)
+    except RequiredValueError:
+      pass
 
-      config.batch_size = 10
-      print(config.batch_size)  # 10
+    config.batch_size = 10
+    print(config.batch_size)  # 10
 
   Args:
     field_type: type of value.
