@@ -1,4 +1,4 @@
-# Copyright 2020 The ML Collections Authors.
+# Copyright 2021 The ML Collections Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,8 @@
 """Tests for ml_collections.ConfigDict."""
 
 import abc
-import collections as python_collections
+from collections import abc as collections_abc
+import functools
 import json
 import pickle
 import sys
@@ -57,7 +58,7 @@ class _TestClass(six.with_metaclass(abc.ABCMeta, object)):
 _test_object = _TestClass()
 
 
-class _TestClassNoStr():  # pylint: disable=old-style-class
+class _TestClassNoStr():
   pass
 
 
@@ -290,6 +291,28 @@ class ConfigDictTest(parameterized.TestCase):
     self.assertEqual(cfg.long_field, int_value)
     self.assertIsInstance(cfg.long_field, expected)
 
+  def testOverrideCallable(self):
+    """Test that overriding a callable with a callable works."""
+
+    class SomeClass:
+
+      def __init__(self, x, power=1):
+        self.y = x**power
+
+      def factory(self, x):
+        return SomeClass(self.y + x)
+
+    fn1 = SomeClass
+    fn2 = lambda x: SomeClass(x, power=2)
+    fn3 = functools.partial(SomeClass, power=3)
+    fn4 = SomeClass(4.0).factory
+    cfg = ml_collections.ConfigDict()
+    for orig in [fn1, fn2, fn3, fn4]:
+      for new in [fn1, fn2, fn3, fn4]:
+        cfg.fn_field = orig
+        cfg.fn_field = new
+        self.assertEqual(cfg.fn_field, new)
+
   def testOverrideFieldReference(self):
     """Test overriding with FieldReference objects."""
     cfg = ml_collections.ConfigDict()
@@ -589,7 +612,7 @@ class ConfigDictTest(parameterized.TestCase):
       hash(cfg)
 
     # Ensure Python realizes ConfigDict is not hashable.
-    self.assertNotIsInstance(cfg, python_collections.Hashable)
+    self.assertNotIsInstance(cfg, collections_abc.Hashable)
 
   def testDidYouMeanFeature(self):
     """Tests 'did you mean' suggestions."""
@@ -675,7 +698,7 @@ class ConfigDictTest(parameterized.TestCase):
   def testDeYaml(self):
     """Tests YAML deserialization."""
     cfg = _get_test_config_dict()
-    deyamled = yaml.load(cfg.to_yaml())
+    deyamled = yaml.load(cfg.to_yaml(), yaml.UnsafeLoader)
     self.assertEqual(cfg, deyamled)
 
   def testJSONConversion(self):
@@ -723,7 +746,7 @@ class ConfigDictTest(parameterized.TestCase):
     cfg_dict.r1 = field
     cfg_dict.r2 = field
 
-    cfg_load = yaml.load(repr(cfg_dict))
+    cfg_load = yaml.load(repr(cfg_dict), yaml.UnsafeLoader)
 
     # Test FieldReferences are preserved
     cfg_load['r1'].set(2)
@@ -1317,6 +1340,21 @@ class ConfigDictUpdateTest(absltest.TestCase):
     self.assertIsInstance(cfg.a, dict)
     self.assertIsInstance(cfg.a['b'], dict)
 
+  def testConvertDictInCopyAndResolveReferences(self):
+    """Test conversion, or not, of dict in copy and resolve references."""
+    cfg = ml_collections.ConfigDict()
+    cfg.a = dict(b=dict(c=0))
+    copied_cfg = cfg.copy_and_resolve_references()
+    self.assertIsInstance(copied_cfg.a, ml_collections.ConfigDict)
+    self.assertIsInstance(copied_cfg.a.b, ml_collections.ConfigDict)
+
+    cfg = ml_collections.ConfigDict(convert_dict=False)
+    cfg.a = dict(b=dict(c=0))
+    copied_cfg = cfg.copy_and_resolve_references()
+    self.assertNotIsInstance(copied_cfg.a, ml_collections.ConfigDict)
+    self.assertIsInstance(copied_cfg.a, dict)
+    self.assertIsInstance(copied_cfg.a['b'], dict)
+
   def testConvertDictTypeCompat(self):
     """Test that automatic conversion to ConfigDict doesn't trigger type errors."""
     cfg = ml_collections.ConfigDict()
@@ -1331,7 +1369,7 @@ class ConfigDictUpdateTest(absltest.TestCase):
     This checks backward compatibility of deserialisation.
     """
     cfg = ml_collections.ConfigDict(dict(a=1))
-    self.assertTrue(yaml.load(cfg.to_yaml())._convert_dict)
+    self.assertTrue(yaml.load(cfg.to_yaml(), yaml.UnsafeLoader)._convert_dict)
 
   def testRecursiveRename(self):
     """Test recursive_rename.
