@@ -1,4 +1,4 @@
-# Copyright 2020 The ML Collections Authors.
+# Copyright 2021 The ML Collections Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,16 @@
 # Lint as: python 3
 """Config flags module."""
 
+from .config_flags import DEFINE_config_dataclass
 from .config_flags import DEFINE_config_dict
 from .config_flags import DEFINE_config_file
+from .config_flags import get_config_filename
+from .config_flags import get_override_values
 
-__all__ = ("DEFINE_config_dict", "DEFINE_config_file")
+__all__ = (
+    "DEFINE_config_dataclass",
+    "DEFINE_config_dict",
+    "DEFINE_config_file",
+    "get_config_filename",
+    "get_override_values",
+)
