@@ -1,4 +1,4 @@
-# Copyright 2020 The ML Collections Authors.
+# Copyright 2021 The ML Collections Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,7 @@
 # limitations under the License.
 
 # Lint as: python 3
-"""Complex configs commmand line parser."""
+"""Configuration commmand line parser."""
 
 import errno
 import imp
@@ -21,9 +21,11 @@ import os
 import re
 import sys
 import traceback
+from typing import Any, Dict, Generic, List, MutableMapping, Optional, Tuple, Type, TypeVar
 
 from absl import flags
 from absl import logging
+import dataclasses
 import ml_collections
 from ml_collections.config_flags import tuple_parser
 import six
@@ -75,12 +77,12 @@ class UnparsedFlagError(flags.Error):
 
 
 def DEFINE_config_file(  # pylint: disable=g-bad-name
-    name,
-    default=None,
-    help_string='path to config file.',
-    flag_values=FLAGS,
-    lock_config=True,
-    **kwargs):
+    name: str,
+    default: Optional[str] = None,
+    help_string: str = 'path to config file.',
+    flag_values: flags.FlagValues = FLAGS,
+    lock_config: bool = True,
+    **kwargs) -> flags.FlagHolder:
   r"""Defines flag for `ConfigDict` files compatible with absl flags.
 
   The flag's value should be a path to a valid python file which contains a
@@ -100,95 +102,78 @@ def DEFINE_config_file(  # pylint: disable=g-bad-name
 
   Typical usage example:
 
-  `script.py`:
+  `script.py`::
 
-  ```python
-  ...
-  from absl import flags
-  from ml_collections.config_flags import config_flags
+    from absl import flags
+    from ml_collections.config_flags import config_flags
 
-  FLAGS = flags.FLAGS
-  config_flags.DEFINE_config_file('my_config')
-  ...
+    FLAGS = flags.FLAGS
+    config_flags.DEFINE_config_file('my_config')
 
-  print(FLAGS.my_config)
-  ```
+    print(FLAGS.my_config)
 
-  `config.py`:
+  `config.py`::
 
-  ```python
-  def get_config():
-    return {
-        'field1': 1,
-        'field2': 'tom',
-        'nested': {
-            'field': 2.23,
-        },
-    }
-  ```
+    def get_config():
+      return {
+          'field1': 1,
+          'field2': 'tom',
+          'nested': {
+              'field': 2.23,
+          },
+      }
 
-  The following command:
+  The following command::
 
-  ```
-  python script.py -- --my_config=config.py
-                      --my_config.field1 8
-                      --my_config.nested.field=2.1
-  ```
+    python script.py -- --my_config=config.py
+                        --my_config.field1 8
+                        --my_config.nested.field=2.1
 
-  will print:
+  will print::
 
-  ```
-  {'field1': 8, 'field2': 'tom', 'nested': {'field': 2.1}}
-  ```
+    {'field1': 8, 'field2': 'tom', 'nested': {'field': 2.1}}
 
   It is possible to parameterise the get_config function, allowing it to
   return a differently structured result for different occasions. This is
   particularly useful when setting up hyperparameter sweeps across various
   network architectures.
 
-  `parameterised_config.py`:
+  `parameterised_config.py`::
 
-  ```python
-  def get_config(config_string):
-    possible_configs = {
-        'mlp': {
-            'constructor': 'snt.nets.MLP',
-            'config': {
-                'output_sizes': (128, 128, 1),
-            }
-        },
-        'lstm': {
-            'constructor': 'snt.LSTM',
-            'config': {
-                'hidden_size': 128,
-                'forget_bias': 1.0,
-            }
-        }
-    }
-    return possible_configs[config_string]
-  ```
+    def get_config(config_string):
+      possible_configs = {
+          'mlp': {
+              'constructor': 'snt.nets.MLP',
+              'config': {
+                  'output_sizes': (128, 128, 1),
+              }
+          },
+          'lstm': {
+              'constructor': 'snt.LSTM',
+              'config': {
+                  'hidden_size': 128,
+                  'forget_bias': 1.0,
+              }
+          }
+      }
+      return possible_configs[config_string]
 
   If a colon is present in the command line override for the config file,
   everything to the right of the colon is passed into the get_config function.
-  The following command lines will both function correctly:
+  The following command lines will both function correctly::
 
-  ```
-  python script.py -- --my_config=parameterised_config.py:mlp
-                      --my_config.config.output_sizes="(256,256,1)"
-  ```
+    python script.py -- --my_config=parameterised_config.py:mlp
+                        --my_config.config.output_sizes="(256,256,1)"
 
-  ```
-  python script.py -- --my_config=parameterised_config.py:lstm
-                      --my_config.config.hidden_size=256
-  ```
+
+    python script.py -- --my_config=parameterised_config.py:lstm
+                        --my_config.config.hidden_size=256
 
   The following will produce an error, as the hidden_size flag does not
-  exist when the "mlp" config_string is provided.
+  exist when the "mlp" config_string is provided::
 
-  ```
-  python script.py -- --my_config=parameterised_config.py:mlp
-                      --my_config.config.hidden_size=256
-  ```
+    python script.py -- --my_config=parameterised_config.py:mlp
+                        --my_config.config.hidden_size=256
 
   Args:
     name: Flag name, optionally including extra config after a colon.
@@ -200,6 +185,9 @@ def DEFINE_config_file(  # pylint: disable=g-bad-name
     lock_config: If set to True, loaded config will be locked through calling
         .lock() method on its instance (if it exists). (default: True)
     **kwargs: Optional keyword arguments passed to Flag constructor.
+
+  Returns:
+    a handle to defined flag.
   """
   parser = _ConfigFileParser(name=name, lock_config=lock_config)
   flag = _ConfigFlag(
@@ -214,17 +202,17 @@ def DEFINE_config_file(  # pylint: disable=g-bad-name
   # Get the module name for the frame at depth 1 in the call stack.
   module_name = sys._getframe(1).f_globals.get('__name__', None)  # pylint: disable=protected-access
   module_name = sys.argv[0] if module_name == '__main__' else module_name
-  flags.DEFINE_flag(flag, flag_values, module_name=module_name)
+  return flags.DEFINE_flag(flag, flag_values, module_name=module_name)
 
 
 def DEFINE_config_dict(  # pylint: disable=g-bad-name
-    name,
-    config,
-    help_string='ConfigDict instance.',
-    flag_values=FLAGS,
-    lock_config=True,
-    **kwargs):
-  """Defines flag for inline `ConfigDict`s compatible with absl flags.
+    name: str,
+    config: ml_collections.ConfigDict,
+    help_string: str = 'ConfigDict instance.',
+    flag_values: flags.FlagValues = FLAGS,
+    lock_config: bool = True,
+    **kwargs) -> flags.FlagHolder:
+  """Defines flag for inline `ConfigDict's` compatible with absl flags.
 
   Similar to `DEFINE_config_file` except the flag's value should be a
   `ConfigDict` instead of a path to a file containing a `ConfigDict`. After the
@@ -233,46 +221,39 @@ def DEFINE_config_dict(  # pylint: disable=g-bad-name
 
   Typical usage example:
 
-  `script.py`:
+  `script.py`::
 
-  ```python
-  ...
-  from absl import flags
+    from absl import flags
 
-  import ml_collections
-  from ml_collections.config_flags import config_flags
+    import ml_collections
+    from ml_collections.config_flags import config_flags
 
 
-  config = ml_collections.ConfigDict({
-      'field1': 1,
-      'field2': 'tom',
-      'nested': {
-          'field': 2.23,
-      }
-  })
+    config = ml_collections.ConfigDict({
+        'field1': 1,
+        'field2': 'tom',
+        'nested': {
+            'field': 2.23,
+        }
+    })
 
 
-  FLAGS = flags.FLAGS
-  config_flags.DEFINE_config_dict('my_config', config)
-  ...
+    FLAGS = flags.FLAGS
+    config_flags.DEFINE_config_dict('my_config', config)
+    ...
 
-  print(FLAGS.my_config)
-  ```
+    print(FLAGS.my_config)
 
-  The following command:
+  The following command::
 
-  ```
-  python script.py -- --my_config.field1 8
-                      --my_config.nested.field=2.1
-  ```
+    python script.py -- --my_config.field1 8
+                        --my_config.nested.field=2.1
 
-  will print:
+  will print::
 
-  ```
-  field1: 8
-  field2: tom
-  nested: {field: 2.1}
-  ```
+    field1: 8
+    field2: tom
+    nested: {field: 2.1}
 
   Args:
     name: Flag name.
@@ -284,6 +265,9 @@ def DEFINE_config_dict(  # pylint: disable=g-bad-name
     lock_config: If set to True, loaded config will be locked through calling
         .lock() method on its instance (if it exists). (default: True)
     **kwargs: Optional keyword arguments passed to Flag constructor.
+
+  Returns:
+    a handle to defined flag.
   """
   if not isinstance(config, ml_collections.ConfigDict):
     raise TypeError('config should be a ConfigDict')
@@ -300,10 +284,107 @@ def DEFINE_config_dict(  # pylint: disable=g-bad-name
   # Get the module name for the frame at depth 1 in the call stack.
   module_name = sys._getframe(1).f_globals.get('__name__', None)  # pylint: disable=protected-access
   module_name = sys.argv[0] if module_name == '__main__' else module_name
-  flags.DEFINE_flag(flag, flag_values, module_name=module_name)
+  return flags.DEFINE_flag(flag, flag_values, module_name=module_name)
 
 
-class _IgnoreFileNotFoundAndCollectErrors(object):
+# Note that we would add a bound to constrain this to be a dataclass, except
+# that dataclasses don't have a specific base class, and structural typing for
+# attributes is currently (2021Q1) not supported in pytype (b/150927776).
+_T = TypeVar('_T')
+
+
+class _TypedFlagHolder(flags.FlagHolder, Generic[_T]):
+  """A typed wrapper for a FlagHolder."""
+
+  def __init__(self, flag: flags.FlagHolder, ctor: Type[_T]):
+    self._flag = flag
+    self._ctor = ctor
+
+  @property
+  def value(self) -> _T:
+    return self._ctor(**self._flag.value)
+
+  @property
+  def default(self) -> _T:
+    return self._ctor(**self._flag.default)
+
+  @property
+  def name(self) -> str:
+    return self._flag.name
+
+
+def DEFINE_config_dataclass(  # pylint: disable=invalid-name
+    name: str,
+    config: _T,
+    help_string: str = 'Configuration object. Must be a dataclass.',
+    flag_values: flags.FlagValues = FLAGS,
+    **kwargs,
+) -> _TypedFlagHolder[_T]:
+  """Defines a typed (dataclass) flag-overrideable configuration.
+
+  Similar to `DEFINE_config_dict` except `config` should be a `dataclass`.
+
+  Args:
+    name: Flag name.
+    config: A user-defined configuration object. Must be built via `dataclass`.
+    help_string: Help string to display when --helpfull is called.
+    flag_values: FlagValues instance used for parsing.
+    **kwargs: Optional keyword arguments passed to Flag constructor.
+  Returns:
+    A handle to the defined flag.
+  """
+
+  if not dataclasses.is_dataclass(config):
+    raise ValueError('Configuration object must be a `dataclass`.')
+
+  # Convert to configdict *without* recursing into leaf node(s).
+  # If our config contains dataclasses (or other types) as fields, we want to
+  # preserve them; dataclasses.asdict recursively turns all fields into dicts.
+  dictionary = {field.name: getattr(config, field.name)
+                for field in dataclasses.fields(config)}
+  config_dict = ml_collections.ConfigDict(initial_dictionary=dictionary)
+
+  # Define the flag.
+  config_flag = DEFINE_config_dict(
+      name,
+      config=config_dict,
+      help_string=help_string,
+      flag_values=flag_values,
+      **kwargs,
+  )
+
+  return _TypedFlagHolder(flag=config_flag, ctor=config.__class__)
+
+
+def get_config_filename(config_flag) -> str:  # pylint: disable=g-bad-name
+  """Returns the path to the config file given the config flag.
+
+  Args:
+    config_flag: The flag instance obtained from FLAGS, e.g. FLAGS['config'].
+
+  Returns:
+    the path to the config file.
+  """
+  if not is_config_flag(config_flag):
+    raise TypeError('expect a config flag, found {}'.format(type(config_flag)))
+  return config_flag.config_filename
+
+
+def get_override_values(config_flag) -> Dict[str, Any]:  # pylint: disable=g-bad-name
+  """Returns a flat dict containing overridden values from the config flag.
+
+  Args:
+    config_flag: The flag instance obtained from FLAGS, e.g. FLAGS['config'].
+
+  Returns:
+    a flat dict containing overridden values from the config flag.
+  """
+  if not is_config_flag(config_flag):
+    raise TypeError('expect a config flag, found {}'.format(type(config_flag)))
+  return config_flag.override_values
+
+
+class _IgnoreFileNotFoundAndCollectErrors:
   """Helps recording "file not found" exceptions when loading config.
 
   Usage:
@@ -315,14 +396,14 @@ class _IgnoreFileNotFoundAndCollectErrors(object):
   """
 
   def __init__(self):
-    self._attempts = []  # type: List[Tuple[Tuple[Text, Text], IOError]]
+    self._attempts = []  # type: List[Tuple[Tuple[str, str], IOError]]
 
   def Attempt(self, description, path):
     """Creates a context manager that routes exceptions to this class."""
     self._current_attempt = (description, path)
     ignore_errors = self
 
-    class _ContextManager(object):
+    class _ContextManager:
 
       def __enter__(self):
         return self
@@ -346,7 +427,7 @@ class _IgnoreFileNotFoundAndCollectErrors(object):
         for attempt, e in self._attempts)
 
 
-def _LoadConfigModule(name, path):
+def _LoadConfigModule(name: str, path: str):
   """Loads a script from external file specified by path.
 
   Unprefixed path is looked for in the current working directory using
@@ -377,7 +458,7 @@ def _LoadConfigModule(name, path):
       name, ignoring_errors.DescribeAttempts()))
 
 
-class _ErrorConfig(object):
+class _ErrorConfig:
   """Dummy ConfigDict that raises an error on any attribute access."""
 
   def __init__(self, error):
@@ -531,7 +612,7 @@ class _ConfigFlag(flags.Flag):
     if self._IsConfigSpecified(sys.argv):
       self.default = default
     else:
-      super(_ConfigFlag, self)._set_default(default)
+      super(_ConfigFlag, self)._set_default(default)  # pytype: disable=attribute-error
     self.default_as_str = "'{}'".format(default)
 
   def _parse(self, argument):
@@ -593,7 +674,8 @@ class _ConfigFlag(flags.Flag):
 
     FLAGS['my_config'].config_filename
 
-    will output 'ml_collections/config_flags/tests/configdict_config.py'
+    will output
+    'ml_collections/config_flags/tests/configdict_config.py'
     ```
 
     Returns:
@@ -684,7 +766,7 @@ def is_config_flag(flag):  # pylint: disable=g-bad-name
   return isinstance(flag, _ConfigFlag)
 
 
-class _ConfigFieldParser(object):
+class _ConfigFieldParser(flags.ArgumentParser):
   """Parser with config update after parsing.
 
   This class-based wrapper creates a new object, which uses
@@ -693,7 +775,12 @@ class _ConfigFieldParser(object):
   the config object.
   """
 
-  def __init__(self, parser, path, config, override_values):
+  def __init__(
+      self,
+      parser: flags.ArgumentParser,
+      path: str,
+      config: ml_collections.ConfigDict,
+      override_values: MutableMapping[str, Any]):
     """Creates new parser with callback, using existing one to perform parsing.
 
     Args:
@@ -718,6 +805,13 @@ class _ConfigFieldParser(object):
     self._override_values[self._path] = value
     return value
 
+  def flag_type(self) -> str:
+    return self._parser.flag_type()
+
+  @property
+  def syntactic_help(self) -> str:
+    return self._parser.syntactic_help
+
 
 def _ExtractIndicesFromStep(step):
   """Separates identifier from indexes. Example: id[1][10] -> 'id', [1, 10]."""
@@ -782,7 +876,7 @@ def _TakeStep(current, step):
   return _AccessConfig(current, field, indices)
 
 
-def GetValue(path, config):
+def GetValue(path: str, config: ml_collections.ConfigDict):
   """Gets value of a single field."""
   current = config
   for step in path.split('.'):
@@ -817,7 +911,7 @@ def GetTypes(paths, config):
   return [GetType(path, config) for path in paths]
 
 
-def SetValue(path, config, value):
+def SetValue(path: str, config: ml_collections.ConfigDict, value):
   """Sets value of a single field."""
   current = config
   steps = path.split('.')
