@@ -1,4 +1,4 @@
-# Copyright 2020 The ML Collections Authors.
+# Copyright 2021 The ML Collections Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,7 @@
 # limitations under the License.
 
 # Lint as: python 3
-r"""Example of basic DEFINE_flag_dict usage.
+r"""Example of basic DEFINE_config_dict usage.
 
 To run this example:
 python define_config_dict_basic.py -- --my_config_dict.field1=8 \
@@ -21,7 +21,6 @@ python define_config_dict_basic.py -- --my_config_dict.field1=8 \
 """
 
 from absl import app
-from absl import flags
 
 import ml_collections
 from ml_collections.config_flags import config_flags
@@ -33,12 +32,11 @@ config.nested = ml_collections.ConfigDict()
 config.nested.field = 2.23
 config.tuple = (1, 2, 3)
 
-FLAGS = flags.FLAGS
-config_flags.DEFINE_config_dict('my_config_dict', config)
+_CONFIG = config_flags.DEFINE_config_dict('my_config_dict', config)
 
 
 def main(_):
-  print(FLAGS.my_config_dict)
+  print(_CONFIG.value)
 
 
 if __name__ == '__main__':
