@@ -1,4 +1,4 @@
-# Copyright 2020 The ML Collections Authors.
+# Copyright 2021 The ML Collections Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -132,7 +132,7 @@ class _ConfigFlagTestCase(object):
   def assert_equal_configs(self, config1, config2):
     """Checks if two configs are identical."""
     self.assert_subset_configs(config1, config2)
-    self.assert_subset_configs(config2, config1)
+    self.assert_subset_configs(config2, config1)  # pylint: disable=arguments-out-of-order
 
 
 class ConfigFileFlagTest(_ConfigFlagTestCase, parameterized.TestCase):
@@ -695,7 +695,8 @@ class ConfigFileFlagTest(_ConfigFlagTestCase, parameterized.TestCase):
                               _TEST_CONFIG_FILE, integer_override,
                               dictfloat_override))
 
-    config.update_from_flattened_dict(values['test_config'].override_values)
+    config.update_from_flattened_dict(
+        config_flags.get_override_values(values['test_config']))
     self.assertEqual(config['integer'], integer_override)
     self.assertEqual(config['float'], original_float)
     self.assertEqual(config['dict']['float'], dictfloat_override)
@@ -708,7 +709,8 @@ class ConfigFileFlagTest(_ConfigFlagTestCase, parameterized.TestCase):
   def testConfigPath(self, config_file):
     """Test access to saved config file path."""
     values = _parse_flags('./program --test_config={}'.format(config_file))
-    self.assertEqual(values['test_config'].config_filename, config_file)
+    self.assertEqual(config_flags.get_config_filename(values['test_config']),
+                     config_file)
 
 
 def _simple_config():
